#!/usr/bin/env python3

"""
Image aspect ratio cropper

This batch tool crops JPG images in the current folder 
to a desired aspect ratio. You can select the cropping 
area with your mouse.

Keyboard commands:
- x or y: zoom in/out cropping area TODO smooth zoom
- c or space or mouseclick: crop
- n: next image
- 1 or 2 or 3: change aspect ratio to 16:9, 4:3, 3:2(default)
- s: switch aspect TODO not working
- q: quit

Usage: 
    python3 image-aspect-ratio-cropper.py

Print filename to image (bottom left corner):
    python3 image-aspect-ratio-cropper.py print
"""

__author__ = "Lorand Horvath"
__copyright__ = "Copyright 2019, hlorand.hu"
__license__ = "https://choosealicense.com/licenses/gpl-3.0/"
__version__ = "0.1"
__email__ = "email at hlorand dot hu"

import cv2
import glob
import os
import sys

# default aspect ratio
a = 3
b = 2

# jpg save quality
quality = 90

printFilename = False
if len(sys.argv) > 1 and sys.argv[1] == "print":
    printFilename = True

# global variable for the cropping area
croppingarea = [(-1,-1),(-1,-1)]

# selection finished?
cropit = False

size = 50
mousewheeldelta = 0

def set_cropping_area(event, x, y, flags, param):
    global croppingarea, cropit, a, b, size, mousewheeldelta

    
    if event == cv2.EVENT_MOUSEWHEEL:
        mousewheeldelta = mousewheeldelta + 1

        if flags > 0:
            if mousewheeldelta > 10:
                size = size + 1
        else:
            if mousewheeldelta > 10:
                size = size - 1
                

        if mousewheeldelta > 10:
            mousewheeldelta = 0
            print("Size: " + str(size))
            # draw cropping area
            origimage = image.copy()
            cv2.rectangle(origimage, croppingarea[0], croppingarea[1], (0, 255, 0), 20)
            cv2.imshow("image", origimage)
    

    if event == cv2.EVENT_MOUSEMOVE:
        croppingarea = []

        # calculate aspect ratio based on image dimensions
        if image.shape[0] > image.shape[1]:
            aspect_a = round(image.shape[0])
            aspect_b = round(image.shape[0]/a*b)
        else:
            aspect_a = round(image.shape[1]/a*b)
            aspect_b = round(image.shape[1])

        size_a = round(aspect_b/size)
        size_b = round(aspect_a/size)

        if size >= 50:
            size_a = 0
            size_b = 0

        # clamp aspect ratio box into the canvas
        if y > image.shape[0]-aspect_a+2*size_b:
            y = image.shape[0]-aspect_a+2*size_b
        if x > image.shape[1]-aspect_b+2*size_a:
            x = image.shape[1]-aspect_b+2*size_a

        # calculate aspect ratio box size
        if image.shape[0] > image.shape[1]:
            croppingarea.append((x, y))
        else:
            croppingarea.append((x, y))

        if image.shape[0] > image.shape[1]:
            #croppingarea.append((x+aspect_a,image.shape[0]))
            croppingarea.append((x+aspect_b-2*size_a,y+aspect_a-2*size_b))
        else:
            croppingarea.append((x+aspect_b-2*size_a,y+aspect_a-2*size_b))

        # draw cropping area
        origimage = image.copy()
        cv2.rectangle(origimage, croppingarea[0], croppingarea[1], (0, 255, 0), 20)
        cv2.imshow("image", origimage)

    # on mouseclick crop the image
    if event == cv2.EVENT_LBUTTONDOWN:
        cropit = True


# create folders
try:
    os.mkdir(os.getcwd() + "/cropped")
except OSError:
    print ("Cannot create directory named \"cropped\" (already exsists?) ")
try:
    os.mkdir(os.getcwd() + "/original")
except OSError:
    print ("Cannot create directory named \"done\" (already exsists?) ")

# load every jpg in the current directory
print("Current directory: " + os.getcwd())
for img in glob.glob('*.jpg') + glob.glob('*.JPG') + glob.glob('*.jpeg') + glob.glob('*.JPEG'):
    
    image = cv2.imread(img)
    clone = image.copy()

    # create window
    cv2.namedWindow("image",cv2.WINDOW_NORMAL)
    cv2.resizeWindow("image", 700,700)
    cv2.moveWindow("image", 40,40)
    cv2.setMouseCallback("image", set_cropping_area)
    cv2.imshow("image", image)

    # debug info
    print(img)
    print("Original W: " + str(image.shape[1]) + " H: " + str(image.shape[0]))

    while cropit == False:
        key = cv2.waitKey(1) & 0xFF

        # keys 1,2,3 modifies the aspect ratio
        if key == ord("1"):
            a = 16
            b = 9
        if key == ord("2"):
            a = 4
            b = 3
        if key == ord("3"):
            a = 3
            b = 2

        step = 1
        if size > 20 and size <= 50:
            step = 10
        if size >= 9 and size <= 20:
            step = 3

        if key == ord("y") and size > 3:
            size = size - step
            
        if key == ord("x") and size < 50:
            size = size + step

        if key == ord("x") or key == ord("y"):
            print("Size: " + str(size))
            # draw cropping area
            origimage = image.copy()
            cv2.rectangle(origimage, croppingarea[0], croppingarea[1], (0, 255, 0), 20)
            cv2.imshow("image", origimage)

        # key s switches the aspect ratio
        if key == ord("s"):
            a,b = b,a
            
        # quit
        if key == ord("q"):
            exit()

        # crop shortcuts (alternative to mouse click)
        if key == ord("c") or key == ord(" "):
            cropit = True
            break

        # next image
        if key == ord("n"):
            break

    
    if cropit and len(croppingarea) == 2:

        # crop the image
        roi = clone[croppingarea[0][1]:croppingarea[1][1], croppingarea[0][0]:croppingarea[1][0]]
        
        # write filename to the image
        if printFilename == True:
            font = cv2.FONT_HERSHEY_DUPLEX
            filename = img.split(" ")
            scale = 0.02 # this value can be from 0 to 1 (0,1] to change the size of the text relative to the image
            fontScale = min(roi.shape[1],roi.shape[0])/(25/scale)
            lineThickness = 1 if roi.shape[1] < 1000 else 2
            cv2.putText(roi, filename[0], ( round(roi.shape[1]/100), round(roi.shape[0]-roi.shape[0]/100) ), font, fontScale, (0, 255, 255), lineThickness, cv2.LINE_AA)

        cv2.imwrite(os.getcwd() + "/cropped/" + img, roi,[int(cv2.IMWRITE_JPEG_QUALITY), quality])
        cropit = False
        size = 50
        print("New      W: " + str(roi.shape[1]) + " H: " + str(roi.shape[0]) )

        # move image to the "original" folder
        os.rename(os.getcwd() + "/" + img, os.getcwd() + "/original/" + img)
     
    # close all open windows
    cv2.destroyAllWindows()
